# 多項式行列の単因子標準形			elmdiv(M)
# ver.1.0  (2020-12-05)
# begin with:
# import mlde
# coded by Kenji Arisawa
# email: arisawa@aichi-u.ac.jp

# 文献
# https://docs.sympy.org/latest/modules/
# https://docs.python.org/3/

# hand translation from elmdiv.mac

# start python3 and then execute:
"""-
import melmdiv
from melmdiv import *
-"""

# comment to developers
# avoiding to restart python:
lds="importlib.reload(melmdiv);from melmdiv import *"	# _xxx is private to the module
"""-
exec(lds)
# NG:
# def ld():
#  importlib.reload(melmdiv)
# we need to avoid 'elmdiv.py', otherwise name confliction!
-"""

def ld(): print("exec(lds) # do this one")

import importlib	# used only in exec(_L). you may comment out
# look https://docs.python.org/3.6/library/importlib.html#importlib.reload for importlib.reload

#from sympy import *
from sympy import Matrix,eye,expand,det,Poly,symbols,sign,div,factor,degree,zeros,latex

x=symbols('x')

debug= True
verbose=True

#
#	some check points in translation
#
"""-
grep -n '[,;][ ]*$' lde.py
-"""


#
#	some notes on Sympy
#
"""-
(1) Vector
vectors in Sympy are special matrix.
they are either single col or single row matrix.
(2) Matrix in Sympy is very flexible data type
(3) Slice on matrix M is great! very powerful!
-"""

#
#	some generic tools
#

def error(*t):
  raise Exception(*t)

# sympy vector is a special matrix
def vlen(v): # vector length
  s = v.shape
  if min(s) != 1: raise Exception("not a vector")
  return max(s)

# 'Matrix' of sympy is a pain for me
# You might think 'matrix' may be better, 
# however I am afraid the name may conflict with other modules such as numpy.
def Mat(*L):
  return Matrix([*L])

# handy test if x is zero equivalent: zero or zero matrix
# look https://docs.python.org/ja/3/library/functions.html for hasattr
# p is a polynomial or polynomial matrix
# if you wand to test a list p, p = Matrix(1,len(p),p); then is_zero(p)
def is_zero(p,*t):
  eflag = 0
  if len(t) and t[0]==1: eflag = 1
  if eflag and hasattr(p,"expand"): p = expand(p)
  if p == 0: return True
  if not hasattr(p,"is_zero_matrix"): return False
  p = list(p[0:])
  L = sublist(p,lambda z: z == 0)
  if len(L) == len(p): return True
  return False
"""-
f = (x**2+2*x+1)-(x+1)**2
is_zero(f)		# → False
is_zero(f,1)	# → True
M = Mat([f],[0])
is_zero(M)			# → False
is_zero(M,1)		# → True
-"""


# let M be a matrix. then expand(M) generates immutable matrix.
# we cannot set value to them. the m_expand(M) below generates mutable matrix
def m_expand(M):
  return expand(M).as_mutable()

# 'factor' works for polynomials and polynomial lists
# 'factor' does nothing to matrix, so define:
def mat_factor(M):
  if type(M)==type([]): return list(map(lambda x: mat_factor(x),M)) # NG: return factor(M)
  if hasattr(M,"factor"): return factor(M)	# 'factor' might support matrix in future
  return M.applyfunc(lambda x: factor(x))

# it may be wise to have sublist() of Maxima
def sublist(L,f):
  return list(filter(f,L))  # need list(...) for Python3

"""-
L=[5,-3,2,-5,3,-7]
sublist(L,lambda x: x>0)	# [5, 2, 3]
-"""  

# p is a polynomial, not working for a polynomial matrix
def hicoeff(p,x):
  return Poly(p,x).coeffs()[0]

# let M be a polynomial matrix
# returns maximum degree of polynomials in M
def hipow(M,x):
  L = M[0:]
  # L = list(map(lambda p: len(Poly(p,x).all_coeffs())-1,L)) or we have already degree()
  L = list(map(lambda p: degree(p),L))
  return max(L)

"""-
hipow(M8,x)	# → 3
-"""

# tab_gen(M) returns new tablet [S,A,T]
# tab_gen2(M,b) is for linear diophantine equations 
def tab_gen(M):
  nrow,ncol = M.shape
  return [eye(nrow),M.copy(),eye(ncol)]

# multiply c to row n of M
def mat_mulrow(M,n,c):
  M[n,0:]=c*M[n,0:]
  return M	# original M is untouched

#  multiply c to col n of M
def mat_mulcol(M,n,c):
  M[:,n]=c*M[:,n]
  return M	# original M is untouched

# mat_addrow(M,i,L)
# mat_addcol(M,j,L)
def mat_addrow(M,i,L):	# L is row vector
  A = M.copy()
  A[i,0:] = A[i,0:]+L
  return A
  
def mat_addcol(M,j,L):	# L is col vector
  A = M.copy()
  A[:,j] = A[:,j]+L
  return A
  
"""-
L = [1,2,3]
M = Mat([-4,-2,5],[3,5,7])
mat_addrow(M,0,L)  # → Mat([-3,0,8],[3,5,7])
mat_addcol(M,1,[6,1])  # → Mat([-4,4,5],[3,6,7])
-"""

def mat_swaprow(M,i,j):
  M[i,0:],M[j,0:] = M[j,0:],M[i,0:]
  return M
  
def mat_swapcol(M,i,j):
  M[:,i],M[:,j] = M[:,j],M[:,i]
  return M
  
"""-
M = Mat([-4,-2,5],[3,5,7])
mat_swaprow(M,0,1) 	# → Mat([3,5,7],[-4,-2,5]), # M is changed 
mat_swapcol(M,0,1)  # → Mat([-2,-4,5],[5,3,7]), # M is kept 
-"""


# diag() in sympy is clumsy! I want to have diagl(L) for a list L
# instead the function below is much more powerful
def genmatrix(f,nrow,ncol):
  M = zeros(nrow,ncol)
  for i in range(0,nrow):
    for j in range(0,ncol):
      M[i,j] = f(i,j)
  return M

"""-
M = Mat([1,2,3],[4,5,6]) 
nrow,ncol = M.shape
genmatrix(lambda i,j: int(i==j)*M[i,j],nrow,ncol)  # → Matrix([[1, 0, 0],[0, 5, 0]])
-"""

# sympy has div for polynomials
# div(x**3-1,x+1,x)	# → (x**2 - x + 1, -2)
# div() is designed for polynomial. we need third arguments (a generator) for safety
# div(5*x+2,3,x)	→	(5*x/3 + 2/3, 0)
# div(5,3,x)		→  (5/3,0)		# div(5,3) is error
# div() doesn't work for matrix

# sympy has a deprecated module "mod". it is wise not to use the name 'mod'
# pmod(p,d) 	p may be a number, a polynomial, or polynomial matrix 
# note that pmod(0,0) returns 0
# p may be a matrix (not a polynomial matrix)
# sympy already has pdiv in different meaning (i.e, pseudo-division), 
# therefore it may be wise to use different name such as mat_pdiv(M,d)
# https://docs.sympy.org/latest/modules/polys/reference.html
def pmod(p,d):
  if is_zero(p,1): return 0
  if hasattr(p,"is_zero_matrix"): return p.applyfunc(lambda z: div(z,d,x)[1])
  return div(p,d,x)[1]
def pdiv(p,d):
  if d==0: error("# ERROR: pdiv")
  if hasattr(p,"is_zero_matrix"): return p.applyfunc(lambda z: div(z,d,x)[0])
  return div(p,d,x)[0]

"""-
M=Matrix([ [x**2 + 1, x + 3], [x - 1, x**3 - 1] ])
M.applyfunc(lambda x: 2*x)
→	Matrix([
	[2*x**2 + 2,    2*x + 6],
	[   2*x - 2, 2*x**3 - 2]])
pmod(M,x**2+1)
→	Matrix([
	[    0,  x + 3],
	[x - 1, -x - 1]])
-"""

#
#	sorting polynomials
#
"""-
def comp(p):
  cp=Poly(p,x).all_coeffs()
  lcp=len(cp)
  return lcp, cp

p1=3+x+x**3
p2=x+2*x**2+5
p3=2*x**2+2*x+7
p4=5+3*x
L=[p1,p2,p3,p4]
sorted(L,key=comp)
→	[3*x + 5, 2*x**2 + x + 5, 2*x**2 + 2*x + 7, x**3 + x + 3]
-"""

def comp(p):
  cp=Poly(p,x).all_coeffs()
  lcp=len(cp)
  return lcp, cp

# uniq sort that works both for polynomial list or matrix
# we don't need mat_sort()
def usort(X):
  return sorted(set(X),key=comp)

"""-
M=Mat([x**2 + 1, x -1],[x - 1, x**3 - 1])
usort(M)
→	[x - 1, x**2 + 1, x**3 - 1]
-"""

# mat_index(M,a) works for polynomial matrix M and polynomial a
def mat_index(M,a):
  return divmod(M[0:].index(a),M.shape[1])
"""-
M = Mat([4,3],[2,5],[3,5])
mat_index(M,2)
→	(1, 0)
-"""

# translation from elmdiv.mac
# lmod(), mat_min() in elmdiv.mac are not used
# mat_mod(M,d) may be replaced by pmod(M,d)

# mat_getdia(M) may be replaced by M.diagonal() but this this not a list
# mat_getdia(M)	# diagonal elements in M
def mat_getdia(M):
  return list(M.diagonal())
"""-
M = Mat([-4,-2,5],[3,5,7])
mat_getdia(M)	# → [-4,5]
-"""

def mat_subm(M,k):
  return M[k:,k:]


# check if M[i,j]==0 (i>=k, j>=k)
def chk_zero(M,k):
  return is_zero(mat_subm(M,k),1)
  
# mat_pivot(M) returns index of minimal polynomial in M,
# by the "minimal polynomial", I mean that making hicoeff positive and then sort
# which is something like a pivot for getting elementary divisor decomposition
def mat_pivot(M):
  X = M.applyfunc(lambda p: sign(Poly(p,x).coeffs()[0])*p)
  L = usort(X)
  m = L[0] 	# minimum value in X
  if m == 0 :
    if len(L) == 1: return None
    m = L[1]
  L = mat_index(X,m)
  return L
"""-
mat_pivot(M)
M = Mat([-4*x**2,3*x],[-2*x,5],[3*x,5])	 # → (1,1)
M = Mat([-4*x**2,3*x],[-2*x**2+3,5],[3*x,5]) # → (1,1)
-"""


# tab_fix(B,k)		# fix B so that make hicoeff(B[2][k,k],x)>0
def tab_fix(B,k):
  S,A,T=B[0], B[1], B[2]
  c = hicoeff(A[k,k],x)
  if  c != 0 and c != 1:
    A = mat_mulrow(A,k,1/c)
    S = mat_mulrow(S,k,1/c)
  return [S,A,T]

# check if M[i,k]==0 (i != k, i=1,2,...)
def chk_col(M,k):
  v = M.col(k); v[k] = 0
  return is_zero(v,1)

# check if M[k,j]=0 (j != k, j=1,2,...)
def chk_row(M,k):
  v = M.row(k); v[k] = 0
  return is_zero(v,1)

# check if we may increase k
def chk_mod(M,k):
  M1 = mat_subm(M,k)
  if is_zero(M1,1): return True
  if M[k,k] == 0: return False
  M1 = pmod(M1,M[k,k])
  if is_zero(M1,1): return True
  return False



# move the minimum element(>0) in abs(mat_subm(A,k)) to A[k,k]
def step1(B,k):
  B = tab_fix(B,k)
  S,A,T = B[0], B[1], B[2]
  L = mat_pivot(mat_subm(A,k))
  if L==None: return False
  i,j = L
  if i != 0 : A = mat_swaprow(A,k,k+i); S = mat_swaprow(S,k,k+i)
  if j != 0 : A = mat_swapcol(A,k,k+j); T = mat_swapcol(T,k,k+j)
  return [m_expand(S),m_expand(A),m_expand(T)]
  
# make absolute values A[k,k],A[k+1,k],... , S[k,k],S[k+1,k],... smaller using modulo */
def step2(B,k):
  B = tab_fix(B,k)
  S,A,T = B[0],B[1],B[2]
  for n in range(k,A.shape[0]):
    if hicoeff(A[n,k],x) < 0: A[n,0:]=-A[n,0:];S[n,0:]=-S[n,0:]
  L = A[k,0:]
  m = L[k]
  L1 = S[k,0:]
  for n in range(k+1,A.shape[0]):
    q = pdiv(A[n,k],m)
    A[n,0:] = A[n,0:]-q*L
    S[n,0:] = S[n,0:]-q*L1
  return [m_expand(S),m_expand(A),m_expand(T)]

# make absolute values A[k,k+1],A[k,k+2],... , T[k,k+1],T[k,k+2],... smaller using modulo
def step3(B,k):
  B = tab_fix(B,k)
  S,A,T = B[0],B[1],B[2]
  L0 = A[k,0:]
  m = L0[k] 	# m is positive
  L2 = A[:, k]
  L3 = T[:, k]
  for n in range(k+1,vlen(L0)):
    q = pdiv(L0[n],m)
    A = mat_addcol(A,n,-q*L2)   
    T = mat_addcol(T,n,-q*L3)
  return [m_expand(S),m_expand(A),m_expand(T)]
"""-
B = [Mat([2,1],[-5,-2]),Mat([1,-5,17],[0,14,-39]),Mat([1,1,0],[0,0,0],[0,0,1])];
step3(B,0) 
  # → [Mat([2,1],[-5,-2]),Mat([1,0,0],[0,14,-39]),
  # Mat([1,6,-17],[0,0,0],[0,0,1])], OK 
-"""

# fix:  B is assumed that both chk_col(B[1],k) and chk_row(B[1],k) are true.
#  then we can make B[1][k,k] smaller
def step4(B,k):
  B = tab_fix(B,k)
  S,A,T = B[0],B[1],B[2]
  L = mat_pivot(pmod(mat_subm(A,k),A[k,k]))
  if L == None : return False
  q = pdiv(A[L[0],L[1]],A[k,k])	# quotient
  L = (L[0]+k,L[1]+k)
  A = mat_addrow(A,L[0],q*A[k,0:])
  S = mat_addrow(S,L[0],q*S[k,0:])
  A = mat_addcol(A,L[1],-A[:,k])
  T = mat_addcol(T,L[1],-T[:,k])
  A = mat_swaprow(A,L[0],k)
  A = mat_swapcol(A,L[1],k)
  S = mat_swaprow(S,L[0],k)
  T = mat_swapcol(T,L[1],k)
  return [m_expand(S),m_expand(A),m_expand(T)]


# final check
def chk_final(B,M):
  S,A,T = B[0],B[1],B[2]
  L = mat_getdia(A)
  # if len(sublist(L,lambda x: x<0)) >0: return 1 # may be also bug in elmdiv.mac
  if len(sublist(L,lambda p: hicoeff(p,x)<0)) >0: return 1
  if A.is_diagonal() == False: return 2
  m = min(A.shape)
  for n in range(0,m-1):
    if pmod(L[n+1],L[n]) != 0: return 3
  # we need to expand polynomials in the matrix to compare, the flag 1 is for the purpose
  if is_zero(S*M*T - A,1) == False: return 4
  return(0)


#
# some debugging tools
#
def dbg(B,M):
  if not is_zero(B[0]*M*B[2] - B[1],1): return False
  return True


def dprint(str,k,B):
  if verbose: print(str,k+1,B)



# our test matrixes are
M1 = Mat([x,-2,-1],[4,x-6,-2],[-4,4,x])   # 斎藤 p.178
M2 = Mat([x-2,-1,0],[0,x-2,0],[0,0,x-2])  # 斎藤 p.179
M3 = Mat([x-4,0,-1],[-2,x-3,-2],[0,2,x])  # 斎藤 p.179
M4 = x*eye(4)-Mat([0,-1,-1,0],[-1,1,0,1],[2,1,2,-1],[-1,-1,-1,1])  # 斎藤 p.194
M5 = x*eye(4)-Mat([-2,-1,-2,1],[5,3,4,-1],[1,0,1,-1],[-3,-1,-2,2]) # 斎藤 p.201
M6 = x*eye(3)-Mat([-5,-1,3],[22,9,-27],[5,2,-6])  # マリツェフ(1) p.133
M7 = Mat([2*x+1,x**2+x+1],[x-1,x+2]) 
M8 = Mat([3*x**3,2*x+1,x**2+x+1],[x-1,x+2,2*x**2+x-1]) 
M9 = Mat([1,0,0,0],[0,1,0,0],[0,0,0,(-x**2)+2*x-1],[0,0,(-x**2)+2*x-1,(-x**3)+2*x**2-x]) 
M10 = Mat([0]) 
M11 = Mat([0,0],[0,0]) 
M12 = Mat([2,0,0,0],[0,3,0,0]) 
M13 = Mat([0,x+1],[x-1,0]) 
M14 = Mat([x-1,1],[-1,x-1]) 
M15 = Mat([1-x,x**2,x],[x,x,-x],[1+x**2,x**2,-x**2])  # マリツェフ(1) p.114
M16 = x*eye(6)-Mat([2,0,0,0,0,0],[0,2,0,0,0,0],[0,0,2,0,0,0],[0,0,0,3,1,0],[0,0,0,0,3,1],
   [0,0,0,0,0,3]) 
M17 = Mat([1-x,x**2,x],[x**2+1,x,-x],[1+x**2,x**2,-x**2]) 
M18 = Mat([3*x**3,2*x+1,x**2+x+1],[x-1,x+2,2*x**2+x-1],[x**2+x,x-3,2*x**3+x**2+x+3]) 
M19 = Mat([x])
ML = [M1,M2,M3,M4,M5,M6,M7,M8,M9,M10,M11,M12,M13,M14,M15,M16,M17,M18,M19] 


# for our test
def test1(*t):
  if len(t) and t[0]==1:
    for M in ML: print(elmdiv(M),"\n======="); input()
  else:
    for M in ML: print(elmdiv(M),"\n=======")


def elmdiv(M):
  n = min(M.shape)
  B = tab_gen(M)
  dprint("step0",-1,B)
  k = 0
  # don't replace B[1] with A, which behaves differently!
  while k < n and chk_zero(B[1],k)==False:
    X = step1(B,k)
    if X == False: break
    B = X
    dprint("step1",k,B)
    
    while True:
      while chk_col(B[1],k) == False:
        B = step2(B,k)
        dprint("step2",k,B)
        B = step1(B,k)
        dprint("step1",k,B)
      while chk_row(B[1],k) == False:
        B = step3(B,k)
        dprint("step3",k,B)
        B = step1(B,k)
        dprint("step1",k,B)
      if chk_col(B[1],k) == True: break
    
    # we may assume here both chk_col(B[1],k) and chk_row(B[1],k) are true
    if chk_mod(B[1],k): k += 1
    else: # BUG! ned this one. you must also fix mlde.py. examine also lde.mac!
      X = step4(B,k)
      if X==False: break
      B = X
      dprint("step4",k,B)
  r = chk_final(B,M)
  if r != 0 : error("# ERROR: bug!",r)
  return mat_factor(B)
  
# B:elmdiv(M)




#
# 最小多項式
# M must be a square matrix
#
def minpoly(M):
  global verbose
  # we assume the M is produced by elmdiv(M)[1]
  nrow,ncol = M.shape
  if nrow != ncol: error("# ERROR: not a square matrix",M)
  # if we have 'x' in M, we should return with error
  if hipow(M,x) != 0: error("# ERROR: M must not include x",M)
  verbose = False
  L = mat_getdia(elmdiv(x*eye(nrow)-M)[1])
  L = sublist(L,lambda x: x!=0)
  # confirmation should be easy on synpy
  # however Poly(x**2,x).eval(M) is error
  # Poly(x**2,x).eval(3) is OK
  # single eval() works for matrix such as eval("M1*M2")
  # you can find some arguments for this topic:
  # https://stackoverflow.com/questions/32445949/substitute-a-sympy-matrix-into-a-polynomial
  # some one says: for polynomial f, f.subs(x,M) works
  return(L[-1])

"""-
confirmation is clumsy for Maxima
for example, minpoly(M) returns x^2-9*x-1 for M:Mat([2,3],[5,7])
we must translate it by hand to x.x-9*x-ident(2) so that it produces zeromatrix for x:M

let M=Mat([0,-1,-1,0],[-1,1,0,1],[2,1,2,-1],[-1,-1,-1,1])
minpoly(M)		# →	(x - 1)**2
minpoly(M).subs(x,M)
→	(-1 + Matrix([
[ 0, -1, -1,  0],
[-1,  1,  0,  1],
[ 2,  1,  2, -1],
[-1, -1, -1,  1]]))**2
expand(minpoly(M).subs(x,M))
→	1 + Matrix([
[-1,  0,  0,  0],
[ 0, -1,  0,  0],
[ 0,  0, -1,  0],
[ 0,  0,  0, -1]])
# if 1 is replaced by 1*eye(4) then the result is correct 
-"""


#
#	行列多項式
#

# generates list of matrix
# note that both matrix in matrix and matrix in array is NG
def mp_gen(M):
  # x must be a symbol
  nrow,ncol = M.shape
  if nrow != ncol: error("# ERROR: not a square matrix")
  L = list(map(lambda p: Poly(p,x).all_coeffs(),M[0:]))
  L1 = list(map(lambda z: len(z),L))
  n = max(L1)  # max degree in M
  L2 = list(map(lambda z: [0]*(n-len(z))+z,L))
  ML = []
  for k in range(0,n):
    ML.append(Matrix(nrow,ncol,list(map(lambda z: z[k],L2))))
  #return Matrix(1,len(ML),ML)
  #print(ML)
  #return Array(ML)
  return ML
  
"""- example:
M = Mat([x**2-2*x-1,2*x+3],[x+2,2*x**2-3*x])
mp_gen(M)
→	[Matrix([
[1, 0],
[0, 2]]), Matrix([
[-2,  2],
[ 1, -3]]), Matrix([
[-1, 3],
[ 2, 0]])]
look 斎藤 p.180 for this example

M = Mat([1,0,0,0],[0,1,0,0],[0,0,0,(-x**2)+2*x-1],[0,0,(-x**2)+2*x-1,(-x**3)+2*x**2-x])
mp_gen(M)
→	[Matrix([
[0, 0, 0,  0],
[0, 0, 0,  0],
[0, 0, 0,  0],
[0, 0, 0, -1]]), Matrix([
[0, 0,  0,  0],
[0, 0,  0,  0],
[0, 0,  0, -1],
[0, 0, -1,  2]]), Matrix([
[0, 0, 0,  0],
[0, 0, 0,  0],
[0, 0, 0,  2],
[0, 0, 2, -1]]), Matrix([
[1, 0,  0,  0],
[0, 1,  0,  0],
[0, 0,  0, -1],
[0, 0, -1,  0]])]
-"""


# translate matrix polynomial Mp to polynomial matrix
# Mp is a array of matrix
def mp_matrix(Mp):
  n = len(Mp)
  L = list(reversed(Mp))
  m = Mp[0].shape[0]
  M = zeros(m,m)
  for k in range(0,n):
    M = M+L[k]*x**k
  return M
  
"""-
M = Mat([x**2-2*x-1,2*x+3],[x+2,2*x**2-3*x])
Mp = mp_gen(M)
mp_matrix(Mp)
→	Matrix([
[x**2 - 2*x - 1,      2*x + 3],
[         x + 2, 2*x**2 - 3*x]])
-"""


def mp_lpad(Mp,n):
  if n <= 0 : return Mp
  nrow,ncol=Mp[0].shape
  return [zeros(nrow,ncol)]*n + Mp


"""- example
Mp=mp_gen(Mat([2*x+1,x**2+x+1],[x-1,x+2]))
mp_lpad(Mp,2)
→	[Matrix([
[0, 0],
[0, 0]]), Matrix([
[0, 0],
[0, 0]]), Matrix([
[0, 1],
[0, 0]]), Matrix([
[2, 1],
[1, 1]]), Matrix([
[ 1, 1],
[-1, 2]])]
-"""

def mp_rpad(Mp,n):
  if n <= 0 : return Mp
  nrow,ncol=Mp[0].shape
  return Mp+[zeros(nrow,ncol)]*n
  
"""- example
Mp=mp_gen(Mat([2*x+1,x**2+x+1],[x-1,x+2]))
mp_rpad(Mp,2)
→	[Matrix([
[0, 1],
[0, 0]]), Matrix([
[2, 1],
[1, 1]]), Matrix([
[ 1, 1],
[-1, 2]]), Matrix([
[0, 0],
[0, 0]]), Matrix([
[0, 0],
[0, 0]])]
-"""
  

def mp_add(Mp1,Mp2):
  n1,n2 = len(Mp1),len(Mp2)
  nrow,ncol=Mp1[0].shape
  if (nrow,ncol) !=Mp2[0].shape: error("# ERROR: Mp1 and MP2 are not same size")
  L1,L2 = Mp1.copy(),Mp2.copy()
  if n1>n2: L2 = mp_lpad(Mp2,n1-n2)
  elif n1<n2: L1 = mp_lpad(Mp1,n2-n1)
  n = max(n1,n2)
  return list(map(lambda i: L1[i]+L2[i],range(0,n)))

def mp_sub(Mp1, Mp2):
  L = list(map(lambda i: -Mp2[i], range(0,len(Mp2))))
  return mp_add(Mp1,L)

"""- example
Mp1=mp_gen(Mat([2*x+1,x**2+x+1],[x-1,x+2]))
Mp2=mp_gen(Mat([2*x**2+1,3*x**2+1],[x**3-1,x**2+2]))
mp_add(Mp1,Mp2)
→ [Matrix([
[0, 0],
[1, 0]]), Matrix([
[2, 4],
[0, 1]]), Matrix([
[2, 1],
[1, 1]]), Matrix([
[ 2, 2],
[-2, 4]])]
-"""

def mp_mul(Mp1,Mp2):
  return mp_gen(expand(mp_matrix(Mp1)*mp_matrix(Mp2)))


# right devision of Mp1 by Mp2
def mp_rdivmod(Mp1,Mp2):
  n1,n2 = len(Mp1),len(Mp2)
  if n1 < n2: return Mp1
  L1,L2 = Mp1.copy(),Mp2.copy()
  MC = L2[0]
  if MC.det() == 0: error("# ERROR: division",MC),
  IMC = MC.inv()
  for k in range(0,n2): L2[k] = IMC*L2[k]
  nrow,ncol = MC.shape
  Q = []
  for k in range(0,n1-n2+1):
    Q.append(L1[0]*IMC)
    L = list(map(lambda i: L1[0]*L2[i],range(0,len(L2))))
    L = mp_rpad(L,(n1-n2-k))
    L1 = mp_sub(L1,L)
    L1 = list(map(lambda i: L1[i], range(1,len(L1))))
  # confirmation
  if not is_zero(expand((mp_matrix(Q)*mp_matrix(Mp2)+mp_matrix(L1)) - mp_matrix(Mp1))):
    return("# ERROR: bug!")
  return Q,L1

"""- example
Mp1=mp_gen(Mat([2*x**2+1,3*x**3+1],[x**3-1,x**2+2]))
Mp2=mp_gen(Mat([2*x+1,x**2+x+1],[x**2-1,x+2]))
mp_rdivmod(Mp1,Mp2)
→ ([Matrix([
[3, 0],
[0, 1]]), Matrix([
[-3, -4],
[ 0,  0]])], [Matrix([
[3,  4],
[1, -2]]), Matrix([
[ 0, 12],
[-1,  2]])])
-"""

# left devision of Mp1 by Mp2
def mp_ldivmod(Mp1,Mp2):
  n1,n2 = len(Mp1),len(Mp2)
  if n1 < n2: return Mp1
  L1,L2 = Mp1.copy(),Mp2.copy()
  MC = L2[0]
  if MC.det() == 0: error("# ERROR: division",MC),
  IMC = MC.inv()
  for k in range(0,n2): L2[k] = L2[k]*IMC
  nrow,ncol = MC.shape
  Q = []
  for k in range(0,n1-n2+1):
    Q.append(IMC*L1[0])
    L = list(map(lambda i: L2[i]*L1[0],range(0,len(L2))))
    L = mp_rpad(L,(n1-n2-k))
    L1 = mp_sub(L1,L)
    L1 = list(map(lambda i: L1[i], range(1,len(L1))))
  # confirmation
  if not is_zero(expand((mp_matrix(Mp2)*mp_matrix(Q)+mp_matrix(L1)) - mp_matrix(Mp1))):
    return("# ERROR: bug!")
  return Q,L1

# mat_trans(M1,M2) get translation matrix P from matrix M1 to M2
# that satisfies M2==P.inv()*M1*P
def mat_trans(M1,M2):
  global verbose
  # if we have 'x' in M, we should return with error
  if hipow(M1,x) != 0 or hipow(M2,x) != 0: error("# ERROR: M must not include x")
  verbose = False
  xM1 = x*eye(M1.shape[0])-M1; xM2 = x*eye(M2.shape[0])-M2
  B1 = elmdiv(xM1); B2 = elmdiv(xM2)
  if B1[1] != B2[1]: return False
  # look マリツェフ(1) p.132 if you want to understand codes below
  T = B1[2]*B2[2].inv()
  P = mp_matrix(mp_rdivmod(mp_gen(T),mp_gen(xM2))[1])
  if P.inv()*M1*P != M2: return("# ERROR: bug!") 	# confirmation
  return P

"""- example
mat_trans(Mat([1,-1],[0,1]),Mat([1,2],[0,1]))
→	Matrix([
[1,  0],
[0, -2]])
M=Mat([0,-1,-1,0],[-1,1,0,1],[2,1,2,-1],[-1,-1,-1,1])
J=Mat([1,1,0,0],[0,1,0,0],[0,0,1,1],[0,0,0,1])
mat_trans(M,J)
→	Matrix([
[ 1,  0,  0,  0],
[-1, -1, -1,  0],
[ 0,  0,  1,  0],
[ 1, -1,  0, -1]])
-"""


"""-
#
# Jordan normal form
# https://docs.sympy.org/latest/modules/matrices/matrices.html
#

# M.jordan_form() returns (P,J) such that J == P.inv()*M*P

M=Mat([6, -3, -2],[4, -1, -2],[3, -2, 0])
M.jordan_form()
→	(Matrix([
[1,   -1, 1/2],
[1,   -1,   1],
[1, -1/2,   0]]), Matrix([
[1, 0, 0],
[0, 2, 1],
[0, 0, 2]]))

M=Mat(
[1, 2, 3],
[4, 5, 6],
[7, 8, 9])
M.jordan_form()
→	(Matrix([
[ 1, (-233 + 39*sqrt(33))/(-803 + 141*sqrt(33)), (39*sqrt(33) + 233)/(803 + 141*sqrt(33))],
[-2,      (-34 + 6*sqrt(33))/(-55 + 9*sqrt(33)),      (34 + 6*sqrt(33))/(9*sqrt(33) + 55)],
[ 1,                                          1,                                        1]]), Matrix([
[0,                   0,                   0],
[0, 15/2 - 3*sqrt(33)/2,                   0],
[0,                   0, 15/2 + 3*sqrt(33)/2]]))

Note: the elmdiv(M)[1] is
Matrix([
[1, 0,                    0],
[0, 1,                    0],
[0, 0, x*(x**2 - 15*x - 18)]])

solve(x*(x**2 - 15*x - 18))
→ [0, 15/2 - 3*sqrt(33)/2, 15/2 + 3*sqrt(33)/2]

for matrix M=Mat([-5,-1,4],[22,8,-11],[5,1,-6]), the M.jordan_form() takes long long long time 
I couldn't measure the time!
the elmdiv(M)[1] is
→	Matrix([
[1, 0,                         0],
[0, 1,                         0],
[0, 0, x**3 + 3*x**2 - 45*x - 36]])
solve(x**3 + 3*x**2 - 45*x - 36)
→	[-1 - 48/((-1/2 - sqrt(3)*I/2)*(297/2 + 81*sqrt(1807)*I/2)**(1/3)) - (-1/2 - sqrt(3)*I/2)*(297/2 + 81*sqrt(1807)*I/2)**(1/3)/3, -1 - (-1/2 + sqrt(3)*I/2)*(297/2 + 81*sqrt(1807)*I/2)**(1/3)/3 - 48/((-1/2 + sqrt(3)*I/2)*(297/2 + 81*sqrt(1807)*I/2)**(1/3)), -1 - (297/2 + 81*sqrt(1807)*I/2)**(1/3)/3 - 48/(297/2 + 81*sqrt(1807)*I/2)**(1/3)]

Of coouse, it is not always possible to have the algebraic representation of the solutions
It will be desirable we have a concept that is something like generalized Jordan matrix
JORDAN AND GENERALIZED JORDAN FORM OF A MATRIX
look http://www.math.lsa.umich.edu/~speyer/593/JordanNormalFormWorksheet.pdf
-"""

